$chip(6502)
$title(Impossible Mission 2)
$paginate
$indent=8
**********************************************
*					     *
*	      Main code 		     *
*					     *
**********************************************

	%include macro.inc
	%include prmseg.inc

	EXTERN	wr_main, wr_aux, rd_main, rd_aux
	EXTERN	spr_1, staani
	EXTERN	scanlo, scanhi	   ;used to find collisions
	EXTERN	lifts, N_LIFTS
	EXTERN	dorkimg(page0), dorknim(page0), dorkx(page0), dorky(page0)
	EXTERN	side(page0)

	public	collide, onlift, liftnum

	SEG	PAGE0

	single	collide 	;result of collision detection
		;on floor/in floor/-/facing Y+5..Y+1
	single	onlift		;Not zero -> on lift
	single	liftnum 	;number of lift if onlift true

	SEG	temps
	single	collx		;X coordinate of where to check collision
	single	colly		;Y coordinate of where to check collision
	single	coltmp		;temp - mainly for collision info
	single	result		;for die if in wall
	double	pt1

	SEG	maincode

;**************************************************************************
;*	COLLISION DETECTION
;**************************************************************************
	public	colldet 	;note not colltst
;-----------------------------
;	check collision table
;-----------------------------
colltst PROC
;translate Y into value for collision table
	lda	colly
	clc
	adc	#4		;offset for actual floor
	and	#$f8
	tay
	cmp	#198		;do not process if off screen
	bcs	outsid
;get x for use as an index
	lda	collx
	lsr	a
	tax
	cmp	#40
	bcs	outsid
	lda	scanlo,y	;use scan table to find collision position
	sta	pt1
	lda	scanhi,y
	clc
	adc	#$04
	sta	pt1+1
	txa
	tay
	sta	rd_main
	lda	(pt1),y       ;get value at the pt1
	sta	rd_aux
	rts
outsid	lda	#0		;off screen returns 0
	rts
colltst ENDPROC

colldet PROC
;-------------------------------------------------------------------------
;	COLLISION DETECTION
;-------------------------------------------------------------------------
	lda	#0
	sta	result

	lda	dorkimg
	asl	a		;use to index staani
	asl	a
	asl	a
	tax
	lda	staani+7,x	;set next animation frame
	sta	dorknim

	lda	#0		;clear collisions
	sta	collide
;check for floor immediately below
	lda	dorky		;copy coordinates to collision area
	sta	colly
	lda	dorkx
	sta	collx
	jsr	colltst
	tax			;save value
	and	#$80		;isolate solid bit
	ora	collide
	sta	collide 	;save

	txa			;restore
	and	#$40		;check for onlift
	sta	onlift
;KLUDGE - removed;
	beq	noflft
;K-r
	lda	side		;check for corridor elevator
	beq	ckwlf
	jmp	noflft1
ckwlf
;find which lift we are standing on
	sta	rd_main
	lda	#0
	sta	coltmp
	ldx	#0
lfsclp
	lda	dorky		;convert graphic Y to character Y
	adc	#4
	lsr	a
	lsr	a
	lsr	a
	ldy	lifts+2 	;check H/V
	beq	hyplt
	ldy	lifts+4,x	;total number of platforms for lift
	jmp	platlp
hyplt	ldy	#1		;1 vertical platform for horizontal
platlp
	cmp	lifts+1,x	;check Y coordinate of lift
	beq	plfnd		;exit if found
	sec
	sbc	lifts+5,x	;subtract interplatform dist (from dork Yc)
	dey
	bne	platlp		;try next platform
	jmp	nxsclf		;try next lift if not this one
plfnd
	lda	dorkx		;confirmed Y coordinate, now check X
	lsr	a		;graphic to character
;horizontal platforms....
	ldy	lifts+2 	;check H/V
	bne	hxplt
	ldy	lifts+4,x	;total number of platforms for lift
	jmp	pltxlp
hxplt	ldy	#1		;1 vertical platform for horizontal
pltxlp
	cmp	lifts,x 	;test each of the three characters
	beq	fndlft		; comprising a lift - found on any match
	sec
	sbc	#1
	cmp	lifts,x
	beq	fndlft
	sec
	sbc	#1
	cmp	lifts,x
	beq	fndlft

	sec
	sbc	lifts+5,x	;subtract interplatform dist (from dork Xc)
	dey
	bne	pltxlp		;try next platform

nxsclf				;not found - index (18 bytes) to next lift
	txa
	clc
	adc	#18
	tax
	inc	coltmp		;next lift number
	lda	coltmp
	cmp	n_lifts 	;at end?
	bne	lfsclp
;lift not found - wierd, for now "forget" we are on a lift
;**********
;KLUDGE
; this will be called each time - the above will be called each
; time and rely on this to correct this
	lda	#0
	sta	onlift
	jmp	noflft1
fndlft
	lda	coltmp		;index points to lift we are on,
	sta	liftnum 	;store in global
noflft
	lda	lifts+2,x	;or H/V - not used yet, but may shorten
	ora	#$40	;KLUDGE
	ora	onlift		;later sections of code
noflft1
	sta	rd_aux
;check if in floor
	lda	dorkimg
	asl	a		;use to index staani
	asl	a
	asl	a
	tax
	lda	staani+5,x	;save current animation frame collision mask
inflp	tax			;save mask
	lda	colly		;move y position up
	sec
	sbc	#8
	sta	colly
	txa			;restore mask
	lsr	a		;shift until first set bit found
	bcc	inflp

	jsr	colltst 	;see if solid
	and	#$80
	clc
	ror	a		;set bit $40 if solid
	ora	collide
	sta	collide

;check if in front of searchable object
	lda	#4
	sta	coltmp
	public	schoblp
schoblp
	jsr	colltst 	;see if solid
	beq	contsr		;general and debug speed
	tay
	and	#$e0
	cmp	#$40		;in front flag
	beq	infrt
	tya
	and	#$a0
	beq	contsr
	lda	coltmp		;in wall or bomb
	cmp	#1
	beq	..inzp3
	tya
	and	#$20		;check for bomb
	beq	..inzp1
	lda	#6		;distinguish bombs
	jmp	..inzp2
..inzp3	tya
	and	#$20		;check for bomb
	beq	contsr
	lda	#6		;distinguish bombs
	jmp	..inzp2
..inzp1
	lda	coltmp		;in wall or bomb
	cmp	#4
	beq	contsr
	tya
	and	#$80		;recheck for wall (in case bomb test)
	beq	contsr
..inzp	lda	#2
..inzp2	sta	result
contsr	dec	coltmp
	beq	schskp
	lda	colly		;move y position up
	sec
	sbc	#8
	sta	colly
	jmp	schoblp
infrt
	lda	#$20
	ora	collide
	sta	collide
schskp
;reset Y displacement for wall check
	lda	dorky
	sec			;start one above floor
	sbc	#8
	sta	colly

;check wall in facing direction
;NOTE: the offset to check left or right must be left alone -
;it must be less or equal than the maximum any animation frame will move
;or walls may be passed right through.	However, if it is any greater
;you will not be able to get right next to a wall (required).
	lda	spr_1+2 	;check direction
	and	#$80
	beq	clft		;alter X based on direction
	lda	collx
	clc
	adc	#2		;add to check right
	sta	collx
	jmp	crght
clft	lda	collx
	sec
	sbc	#2		;subtract to check left
	sta	collx
crght
	lda	#5		;do 5 positions starting above floor
	sta	coltmp
csdlp	jsr	colltst 	;check for collision
	and	#$80
	clc
	ror	a		;offset to proper field
	ror	a
	ldx	coltmp
cbtlp	ror	a		;offset to proper bit in field
	dex
	bne	cbtlp
	ora	collide 	;save
	sta	collide
	lda	colly		;displace Y
	sec
	sbc	#8		;up one character/8 scan lines
	sta	colly
	dec	coltmp
	bne	csdlp
	lda	result
	rts
colldet ENDPROC

	END
