$chip(6502)
$title(Impossible Mission 2)

**********************************************
*		Sprite routines 	     *
**********************************************

	%include macro.inc
	%include secseg.inc

	EXTERN	scanlo,scanhi	     ;used to find collisions
	extern	rd_aux,wr_aux,rd_main,wr_main

	EXTERN	lifts, N_LIFTS
	extern	n_robots, robots, spr_1
	extern	n0rob,n1rob,n2rob,n3robd,n3robu,n4rob,robshot
	extern	robmb,roboom,holes

	extern	srchflg,donesch

	seg	page0

	single	rlftnum
	single	stnflg
	
	single	grabflg		;set when pushbot too near
	single	rdorkx		;save area for passed dorkx
	single	rdorky		;save area for passed dorky
	single	cycle		;phase counter for robot speed
	single	toward		;set when robot faces dork
	single	xtmp		;x (char) coordinate of robot
	single 	ytmp		;y (char*8) coord of robot
	single	xtst		;gp save and x value to test
	single 	ytst		;gp save and y value to test
	single	dtmp		;direction of robot from sprite
	single	ctmp		;gp save
	single	flrtmp		;floor under robot (lift or solid, etc.)
	single	seen		;set if seen
	double	pt1

	public	dorobot,setrob

FOL	equ	1
PAT	equ	2
ZAP	equ	3
PAU	equ	4
ALT	equ	5

OLD	equ	0
PSH	equ	1
LFT	equ	2
TRT	equ	3
BMB	equ	4

	public	robprog,robtrk

	seg	graph

;robot program option flags

LFTBND	equ	$80		;lifts are a boundary (don't cross)
BAKCHK	equ	$40		;occasionally turn back to check follow
ZAPBEF	equ	$20		;zap before turning
ZAPAFT	equ	$10		;zap after turning

;*************************
;
;	these are the prototype programs for the robots.
;	the four bytes are as follows:
;		speed
;		not in sight mode
;		in sight mode
;		option flags	-lift bounds, zap and turnn, etc.
;
;*************************

robprog	db	0,FOL,FOL,$80
	db	2,FOL,FOL,$80
	db	1,FOL,FOL,$80
	db	1,FOL,ZAP,$a0	;ZAP:zap,pause
	db	1,FOL,ZAP,$a0	;ZAP:pause,zap,pause

	db	4,PAT,FOL,$80
	db	2,PAT,FOL,$80
	db	1,PAT,FOL,$80
	db	1,PAT,FOL,$c0	;PAT:chk back
	db	1,PAT,FOL,$c0	;PAT:chk back,FOL:zap
;10
	db	2,ALT,ALT,$80	;zap
	db	2,PAU,PAU,$a0	;zap
	db	1,PAU,PAU,$a0	;zap
	db	1,PAT,ZAP,$80
	db	4,PAT,PAT,$a0	;zap,turn

	db	2,PAT,PAT,$a0	;zap,turn
	db	1,PAT,PAT,$a0	;zap,turn
	db	2,ALT,ALT,$a0	;zap, random turning
	db	2,ALT,PAT,$80	;mutant, spd to 1,2
	db	4,PAT,PAT,$80
;20
	db	2,PAT,PAT,$80
	db	1,PAT,PAT,$80
	db	1,PAT,PAT,$00	;ignore some bounds
	db	2,PAT,PAT,$80	;alt speed 1/2
	db	2,PAT,PAT,$80	;alt speed 1/4

	db	4,ALT,FOL,$80	;pause after alt
	db	4,PAT,PAT,$b0	;zap,turn,zap
	db	2,PAT,PAT,$b0	;zap,turn,zap
	db	1,PAT,PAT,$b0	;zap,turn,zap
	db	1,PAT,PAT,$00	;*lift in sight speed to 4
;30
	db	1,PAT,PAT,$00	;*should be lift turn pattern
	db	1,PAT,PAT,$00	;*drop bomb occasionally
	db	1,ALT,ALT,$00	;*TORT
	db	4,FOL,FOL,$00	;zap,pause occasionally

;*************************
;
;	This buffer holds the actual program.  the first 4 bytes are copied
;	from the program table, and the last byte is the type (push, tort,
;	lift, etc.).  the byte before last is an internal flag.  The length
;	of this table is the same as the sprite definition block to keep
;	indexing easy.
;
;*************************
robtrk	ds	6*8

	seg	graph

;*************************
;
;	This routine does all the setup necessary for the robots
;
;*************************
	public	setrob
setrob	proc
**********************************
*	Horrid Robotic Kludge
**********************************
	sta	wr_aux
	sta	rd_main
	ldx	#0		;clear robot index
	ldy	#0		;clear sprite index
	move	xtmp,n_robots	;count robots
	lbeq	..ero
..rolp4
	lda	robots,x	;adjust collision location to sprite location
	asl	a
;	clc
;	adc	#2
	sta	spr_1+12,y	;set X
	lda	robots+1,x	;get y
	asl	a
	asl	a
	asl	a
	sbc	#20
	sta	spr_1+13,y	;set Y

	lda	#$8a		;set direction right (was left), color
	sta	spr_1+14,y

	lda	#0
	sta	spr_1+15,y	;disable sprite

	lda	#$40
	sta	robtrk,y	;disable robot program (for now)

	lda	robots+2,x	;get program type
	lbmi	..nxrob		;skip programming if disabled
	
	cmp	#33		;see if special handling required
	bcc	oldbots
	sec
	sbc	#33
	cmp	#29
	beq	newbot

	sta	ctmp		;set things for pushbot
	lda	#high(n1rob)
	sta	spr_1+17,y
	lda	#LOW(n1rob)
	sta	spr_1+16,y
	lda	#PSH
	sta	robtrk+5,y
	jmp	setprg

oldbots	sta	ctmp		;special robot type checking
	cmp	#32
	beq	tortbot
	cmp	#31
	beq	bombot
	cmp	#30
 	beq	liftbot
	cmp	#29
	beq	liftbot
	jmp	..sknewb	;set things for standard robot
newbot	lda	#33		;special program oldbot
	sta	ctmp
..sknewb
	lda	#high(n0rob)
	sta	spr_1+17,y
	lda	#LOW(n0rob)
	sta	spr_1+16,y
	lda	#OLD
	sta	robtrk+5,y
	jmp	setprg

tortbot				;set things for tortbot
	sta	rd_aux
	lda	spr_1+12,y	;move x
	sec
	sbc	#2		;(was 4)
	sta	spr_1+12,y
	lda	spr_1+13,y	;move 
	sec
	sbc	#4
	sta	spr_1+13,y
	sta	rd_main
	lda	#high(n3robd)
	sta	spr_1+17,y
	lda	#LOW(n3robd)
	sta	spr_1+16,y
	lda	#TRT
	sta	robtrk+5,y

	jmp	setprg

bombot				;set things for bombot
	lda	#high(n4rob)
	sta	spr_1+17,y
	lda	#LOW(n4rob)
	sta	spr_1+16,y
	lda	#BMB
	sta	robtrk+5,y
	jmp	setprg

liftbot				;set things for liftbot
	lda	#high(n2rob)
	sta	spr_1+17,y
	lda	#LOW(n2rob)
	sta	spr_1+16,y
	lda	#LFT
	sta	robtrk+5,y
	jmp	setprg

setprg				;set the robot program from table

	lda	#$e6
	sta	spr_1+15,y	;enable sprite

	lda	ctmp		;change prog num to index
	asl	a
	asl	a
	stx	ctmp		;save y
	tax			;use new index
	lda	robprog,x
	sta	robtrk,y
	lda	robprog+1,x
	sta	robtrk+1,y
	lda	robprog+2,x
	sta	robtrk+2,y
	lda	robprog+3,x
	sta	robtrk+3,y
	lda	#0
	sta	robtrk+4,y
	ldx	ctmp
..nxrob
	inx			;index to next
	inx
	inx
	iny
	iny
	iny
	iny
	iny
	iny
	dec	xtmp
	lbne	..rolp4
..ero
	lda	#$ff		;set end markers and set sprite endlist
	sta	robtrk,y
	lda	#1
	sta	spr_1+15,y
	sta	rd_main
	sta	wr_main

	rts

setrob	endproc


;*************************
;
;	robot handling
;
;*************************

dorobot	PROC
	sta	stnflg
	stx	rdorkx
	sty	rdorky
	sta	rd_aux
	sta	wr_aux
	
;turn off shots - all color 3 sprites
	ldy	#0
..clshtlp
	lda	spr_1+15,y
	cmp	#1
	beq	..enclsht
	lda	spr_1+14,y
	and	#$7f
	cmp	#$03
	bne	..ntsht
	lda	#0
	sta	spr_1+15,y
..ntsht
	tya
	clc
	adc	#6
	tay
	jmp	..clshtlp
..enclsht
	
	inc	cycle
	lda	#0
	sta	roboom
	sta	robmb
	sta	grabflg
	ldx	#0
..rotop
	lda	robtrk,x
	lbmi	..xit		;exit on negative speed
	cmp	#$40		;skip dead robot
	lbeq	..nxrbt

	lda	spr_1+15,x	;disabled sprite
	lbeq	..nxrbt

;get coordinates
	lda	spr_1+12,x	;get x
	clc
	adc	#2
	sta	xtmp		;save collision X
	lda	spr_1+13,x	;get y
	clc
	adc	#20
	sta	ytmp		;save collision Y
	lda	spr_1+14,x
	sta	dtmp		;get direction

;lift if in floor
	lda	robtrk+5,x
	cmp	#TRT
	lbeq	..nolf		;tort can't lift itself
	lda	xtmp
	sta	xtst
	lda	ytmp
	sec
	sbc	#8
	sta	ytst
	jsr	robcol
	and	#$e0
	lbeq	..nolf

	tay
	and	#$80
	lbne	..lftit
	tya
	and	#$40
	lbne	..nolf

;reset bomb mask (clear bomb)
	lda	ytst
	clc
	adc	#4		;offset for actual floor
	and	#$f8
	tay
	lda	scanlo,y	;use scan table to find collision position
	sta	pt1
	lda	scanhi,y
	clc
	adc	#$04
	sta	pt1+1
	lda	xtst
	lsr	a
	tay
	sta	rd_main
	sta	wr_main
	lda	(pt1),y       ;get value at the pt1
	and	#$df
	sta	(pt1),y       ;get value at the pt1
	sta	rd_aux
	sta	wr_aux


	ldy	#0
..hllp	lda	holes,y
	cmp	#1
	beq	..fndhl
..hllp1	iny
	iny
	iny
	cpy	#24
	bne	..hllp
	jmp	..nxrbt		;???????? didn't find bomb
..fndhl
	lda	xtst
	lsr	a
	cmp	holes+1,y
	bne	..hllp1
	lda	ytst
	clc
	adc	#4		;offset for actual floor
	lsr	a
	lsr	a
	lsr	a
	cmp	holes+2,y
	bne	..hllp1
	lda	#$82
	sta	holes,y

	lda	#1
	sta	roboom

	jmp	..die

..lftit
	lda	ytmp	;lift it
	sec
	sbc	#28
	sta	spr_1+13,x

	jmp	..nxrbt
..nolf

;make sure there is a floor below, else fall
	lda	robtrk+5,x
	cmp	#TRT
	bne	..dofl		;tort can't lift itself
	lda	robtrk+4,x
	and	#$80
	bne	..nofl
..dofl
	lda	xtmp
	sta	xtst
	lda	ytmp
	sta	ytst
	jsr	robcol
	sta	flrtmp
	and	#$80
	lbeq	..fall

..nofl
	lda	robtrk+5,x	;figure type and handle special cases
	cmp	#TRT
	lbeq	dotort
	lda	stnflg
	lbne	..nxrbt

;
	lda	robtrk+5,x	;figure type and handle special cases
	cmp	#LFT
	beq	dolft
	jmp	normbot

	public	dolft
;-------------------------
;lift robot handling
dolft
	lda	flrtmp		;check if on lift
	and	#$40
	lbeq	..nrmbt1	;if not, handle normally (but reset flag)

	jsr	loclft		;which one?
	lbeq	normbot
	jsr	cntrbot		;force center on moving lifts
	cmp	#$ff
	beq	..nrmbt2
	asl	a
	clc
	adc	#1		;was adc 3
	sta	spr_1+12,x
	lda	spr_1+14,x
	and	#$80
	bne	..strtd
	lda	spr_1+12,x
	sec
	sbc	#1
	sta	spr_1+12,x
	jmp	..strtd		;continue check (indicate already started)

..nrmbt2

	sta	rd_main
	lda	srchflg
	sta	rd_aux
	lbne	..nrmbt1

	lda	robtrk+4,x
	and	#1
	bne	..enlft		;don't restart lift just after it stops

	lda	#4		;speed to 4 (to keep up when on hlifts)
	sta	robtrk,x
	lda	robtrk+4,x	;try last direction
	and	#$2
	jsr	strtlft		;start lift

	cpy	#0
	bne	..nohzfix	;face in direction of hlift

	and	#$80
	bne	..hzfix0
	lda	spr_1+14,x
	ora	#$80
	sta	spr_1+14,x
	lda	#1
	jmp	..hzfix1
..hzfix0
	lda	spr_1+14,x
	and	#$7f
	sta	spr_1+14,x
	lda	#$ff
	jmp	..hzfix1
..nohzfix
	cmp	#0
	beq	..strtd
..hzfix1

	and	#$2
	sta	robtrk+4,x	;save started direction
;	jmp	..nxrbt
	cpy	#0
	bne	..strtd
	jsr	cntrbot		;force center on moving lifts
	cmp	#$ff
	beq	..nrmbt2x
	asl	a
	clc
	adc	#$0		;was ff
	sta	spr_1+12,x
..nrmbt2x

..strtd
	lda	robtrk+4,x	;set started flag
	ora	#1
	sta	robtrk+4,x
	jmp	..nxrbt

..nrmbt1			;clear started flag (off lift detect)
	lda	robtrk+4,x
	and	#$fe
	sta	robtrk+4,x
	lda	#1		;reset speed to 1
	sta	robtrk,x
..enlft	jmp	normbot

;-------------------------
;normal robot processing

;determine if facing by comparing coordinates and direction bit
normbot
	lda	robtrk+5,x	;always in sight if not killerbot
	cmp	#PSH+1
	lbcs	..notgrb

	lda	#0
	sta	seen
	lda	rdorkx
	sec
	sbc	xtmp
	clc
	adc	#2	;1,6
	cmp	#4
	bcc	..dirto		;same coordinate implies facing
	lda	rdorkx
	cmp	xtmp
	bcc	..golf		;check to left
	lda	dtmp
	and	#$80
	bne	..dirto
..diraw	lda	#0
	jmp	..dirknw
..golf
	lda	dtmp
	and	#$80
	bne	..diraw
..dirto	lda	#$ff
..dirknw
	sta	toward		;save flag
	beq	..notis

;is facing, determine if seen
	lda	rdorky
	clc
	adc	#24
	sec
	sbc	ytmp
	bmi	..notis
	cmp	#36		;vertical range of sight
	bcs	..notis
	lda	#1
	sta	seen
;in sight, handle grabbing for pushbots
;(also shot collision later)
	lda	rdorkx
	sec
	sbc	xtmp
	clc
	adc	#1	;2
	tay
	lda	dtmp		;check direction
	and	#$80
	bne	..check_push_left
	tya
	cmp	#6		;close?
	bcs	..notgrb
	jmp	..check_near

..check_push_left
	tya
	cmp	#8		;close?
	bcs	..notgrb
	cmp	#1
	bcc	..notgrb
..check_near
	tya
	cmp	#7
	beq	..notdeep
	cmp	#0
	beq	..notdeep
	lda	grabflg		;set dead flag
	ora	#4
	sta	grabflg
	tya
	cmp	#6
	beq	..notdeep
	cmp	#1
	beq	..notdeep
	lda	grabflg		;set dead flag
	ora	#8
	sta	grabflg
..notdeep
	lda	grabflg		;set dead flag
	ora	#2
	sta	grabflg
;set grab flag only for pushbots
	lda	robtrk+5,x
	cmp	#PSH
	bne	..notgrb
	lda	dtmp		;set grab flag with direction
	and	#$80
	ora	#1
	ora	grabflg
	and	#$fd		;remove dead for pushbot
	sta	grabflg
..notgrb

;select mode based on in sight value
	lda	robtrk+2,x
	jmp	..isis
..notis
	lda	robtrk+1,x
..isis
	tay

;ignore rest depending on speed and cycle
	lda	robtrk,x
	lbeq	..fastst	;special handling for zero speed later
	cmp	#4
	beq	..fastst
	cmp	#2
	bne	..slowst
	lda	cycle
	and	#1
	bne	..fastst
	jmp	..nxrbt
..slowst
	lda	cycle
	and	#3
	lbne	..nxrbt
..fastst

	tya
;execute selected program
	cmp	#FOL
	lbeq	follow
	cmp	#PAT
	lbeq	patrol
	cmp	#ALT
	lbeq	altern
	cmp	#ZAP
	beq	dozap
	cmp	#PAU
	beq	dopau
	jmp	..nxrbt		;error if here, but just go to next robot

;.........................
dopau
	lda	robtrk+3,x
	and	#ZAPBEF
	beq	..pse
	lda	robtrk+4,x
	eor	#1
	sta	robtrk+4,x
	and	#1
	beq	dozap
..pse
	jmp	..nxrbt

;.........................
;Zap handling - jumped to from other locations as well
dozap
	txa
	tay
;find next avail sprite slot
..cknxz	lda	spr_1+15,y
	and	#$fe
	beq	..fndz
	tya
	clc
	adc	#6
	tay
	jmp	..cknxz
	
..fndz
	lda	spr_1+15,y	;patch end of table
	and	#1
	beq	..nonend
	lda	#1
	sta	spr_1+15+6,y
..nonend

	lda	#$e6
	sta	spr_1+15,y
	lda	spr_1+13,x
	clc
	adc	#1
	sta	spr_1+13,y
	lda	spr_1+14,x
	and	#$80
	beq	..zplft

	lda	spr_1+12,x
	clc
	adc	#8
	sta	spr_1+12,y

	jmp	..zprgt

..zplft	lda	spr_1+12,x
	sec
	sbc	#7
	sta	spr_1+12,y

..zprgt
	sec
	sbc	rdorkx
	clc
	adc	#6
	cmp	#9
	bcs	..zpnd
	lda	seen
	beq	..zpnd
	lda	grabflg
	ora	#2
	sta	grabflg
..zpnd
	lda	#$03
	sta	spr_1+14,y
	lda	#high(robshot)
	sta	spr_1+17,y
	lda	#low(robshot)
	sta	spr_1+16,y

	jmp	..nxrbt

;.........................
;alternate handling
altern
	lda	robtrk+3,x
	and	#ZAPBEF
	beq	doalt
	lda	robtrk+4,x
	eor	#1
	sta	robtrk+4,x
	and	#1
	lbeq	dozap
doalt
	jmp	..rvrs

;.........................
;Follow handling
follow
	lda	toward		;check if going toward (and force)
	beq	..rvrs
	jsr	godir
	jsr	chkbar
	lbne	..nxrbt		;stop if at boundary
	lda	robtrk,x	;zero speed
	lbeq	..nxrbt
	jmp	ptrl2

;.........................
;patrol handling
patrol
;shift position in direction of move
	jsr	godir
	jsr	chkbar
	bne	..rvrs		;reverse at boundary

;-------------------------
;no problem, move there (follow or patrol)
ptrl2
	lda	robtrk+5,x	;check for zap in oldbot
	bne	..nozpp
	lda	robtrk+4,x	;afterzap required?
	beq	..nozpp
	lda	#0
	sta	robtrk+4,x	;yes, zap and clear flag
	lda	robtrk+3,x
	and	#ZAPAFT
	lbne	dozap
..nozpp
	lda	robtrk,x
	lbeq	..nxrbt		;don't move if zero speed

	lda	robtrk+5,x	;check for zap in oldbot
	cmp	#bmb
	bne	..nodrp
	
	lda	flrtmp		;not on lifts
	and	#$40
	bne	..nodrp
	
	lda	xtmp
	and	#7
	cmp	#4
	bne	..nodrp
	lda	#1
	sta	robmb
	lda	xtmp
	clc
	adc	#2
	sta	robmb+1
	lda	ytmp
	sta	robmb+2
..nodrp
	lda	xtmp
	sec
	sbc	#2
	sta	spr_1+12,x
	jmp	..nxrbt

;-------------------------
;turn around
..rvrs

	lda	robtrk+5,x	;check zapbef for oldbots
	bne	..nozpp1
	lda	robtrk+4,x	;check flag (actually reverse 2nd time thru)
	clc
	adc	#$1
	sta	robtrk+4,x
	cmp	#1
	bne	..nozpp1
	lda	robtrk+3,x
	and	#ZAPBEF
	lbne	dozap
..nozpp1
	lda	robtrk+5,x
	cmp	#LFT
	bne	..nozpp2
	lda	robtrk+4,x	;clear unstart on barrier
	and	#$fe
	sta	robtrk+4,x
..nozpp2
	lda	spr_1+14,x	;complement direction bit
	eor	#$80
	sta	spr_1+14,x

	jmp	..nxrbt

;--------------------------------------------------
..fall
	lda	ytmp
	sec
	sbc	#12
	sta	spr_1+13,x
	cmp	#200		;die if off bottom
	bcc	..norodie
	cmp	#248
	bcs	..norodie	;just above top (so they will fall onscreen)
;alt entry point for bomb encounters
..die
	lda	#$40		;set dead flag
	sta	robtrk,x
	lda	#0
	sta	spr_1+15,x
	
	sta	wr_main
	txa
	lsr	a
	tay
	lda	#$ff
	sta	robots+2,y
	sta	wr_aux
		
..norodie
	jmp	..nxrbt

;--------------------------------------------------
	public	dotort
dotort
	sta	rd_main
	lda	donesch
	sta	rd_aux
	beq	..normrob
	lda	robtrk+4,x
	ora	#$7
	sta	robtrk+4,x	
..normrob
	lda	robtrk+4,x
	clc
	adc	#1
	and	#$f7
	sta	robtrk+4,x
	tay
	and	#$40
	bne	..domid

	lda	stnflg
	bne	..lockifstun

	tya
	and	#7
	lbne	..nxrbt
	lda	robtrk+4,x
	eor	#$80
	sta	robtrk+4,x
	and	#$80
	beq	..trtdn

	lda	robtrk+4,x
	ora	#$40
	sta	robtrk+4,x

	lda	spr_1+13,x
	clc
	adc	#24		;offset for actual floor (down sprite)
	and	#$f8
	sta	ytmp

	lda	#high(n3robu)
	sta	spr_1+17,x
	lda	#LOW(n3robu)
	sta	spr_1+16,x
	lda	spr_1+13,x
	sec
	sbc	#12
	sta	spr_1+13,x
	jmp	..trtup
..trtdn
	lda	spr_1+13,x
	clc
	adc	#36		;offset for actual floor (up sprite)
	and	#$f8
	sta	ytmp

	lda	#high(n3robd)
	sta	spr_1+17,x
	lda	#LOW(n3robd)
	sta	spr_1+16,x
	lda	spr_1+13,x
	clc
	adc	#12
	sta	spr_1+13,x
	jmp	..trtup
..domid
	lda	spr_1+13,x
	clc
	adc	#32		;offset for actual floor (down sprite)
	and	#$f8
	sta	ytmp
	lda	robtrk+4,x
	and	#$bf
	sta	robtrk+4,x
	jmp	..trtup
..lockifstun
	lda	spr_1+13,x
	clc
	adc	#32		;offset for actual floor (down sprite)
	and	#$f8
	sta	ytmp

	lda	robtrk+4,x
	and	#$f0
	sta	robtrk+4,x

..trtup
	lda	#3
	sta	ctmp
;get x for use as an index
..yslp
	lda	ytmp
	sec
	sbc	#8
	sta	ytmp
	tay
	lda	scanlo,y	;use scan table to find collision position
	sta	pt1
	lda	scanhi,y
	clc
	adc	#$04
	sta	pt1+1
	lda	xtmp
	lsr	a
	tay
	sta	rd_main
	sta	wr_main
	dey
	lda	ctmp
	cmp	#3
	beq	..doset
	sta	rd_aux
	lda	robtrk+4,x
	and	#$80
	beq	..doclr
	lda	robtrk+4,x
	and	#$40
	beq	..doset	
	lda	ctmp
	cmp	#1
	beq	..donone	
..doset
	sta	rd_main
	lda	(pt1),y       ;get value at the pt1
	ora	#$80
	sta	(pt1),y       ;get value at the pt1
	iny
	lda	(pt1),y       ;get value at the pt1
	ora	#$80
	sta	(pt1),y       ;get value at the pt1
	iny
	lda	(pt1),y       ;get value at the pt1
	ora	#$80
	sta	(pt1),y       ;get value at the pt1
	jmp	..skswp
..doclr
	sta	rd_main
	lda	(pt1),y       ;get value at the pt1
	and	#$7f
	sta	(pt1),y       ;get value at the pt1
	iny
	lda	(pt1),y       ;get value at the pt1
	and	#$7f
	sta	(pt1),y       ;get value at the pt1
	iny
	lda	(pt1),y       ;get value at the pt1
	and	#$7f
	sta	(pt1),y       ;get value at the pt1
	jmp	..skswp
..donone
..skswp
	sta	rd_aux
	sta	wr_aux
	dec	ctmp
	lbne	..yslp
	jmp	..nxrbt

..nxrbt
	inx
	inx
	inx
	inx
	inx
	inx
	jmp	..rotop
..xit	sta	rd_main
	sta	wr_main
	lda	grabflg
	rts

;**************************************************
;	Robot utility routines
;**************************************************
;move one in direction of sprite facing
godir
	lda	dtmp
	and	#$80
	beq	..lft
	inc	xtmp
	rts
..lft	dec	xtmp
	rts

;**************************************************
chkbar
;check for missing floor or lift
	lda	xtmp
;	cmp	#1
	bmi	..banogo
	cmp	#80
	beq	..banogo

	lda	xtmp
	sta	xtst

	lda	spr_1+14,x
	and	#$80
	beq	..norj1
	lda	xtst
	clc
	adc	#2
	sta	xtst
..norj1
	lda	ytmp
	sta	ytst
	jsr	robcol
	sta	xtst
	and	#$80	;#$c0
;if $c0, indicate lift
	beq	..banogo

	lda	robtrk+3,x	;some robots stop before lifts
	and	#$80
	beq	..nolbnd
	lda	robtrk+5,x	;some robots stop before lifts
	cmp	#LFT
	beq	..nflfr

	lda	flrtmp
	and	#$40
	bne	..nolbnd	;get off lifts
..nflfr

	lda	xtst
	and	#$40
	bne	..banogo
..nolbnd

;check for wall
	lda	xtmp
	sta	xtst
	lda	spr_1+14,x
	and	#$80
	beq	..norj2
	lda	xtst
	clc
	adc	#2
	sta	xtst
..norj2
	lda	ytmp
	sec
	sbc	#8
	sta	ytst
	jsr	robcol
	and	#$80
	bne	..banogo
	lda	#0
	rts
..banogo
	cmp	#LFT
	bne	..nflfl
	lda	robtrk+4,x
	and	#$fe
	sta	robtrk+4,x
..nflfl
	lda	#$80
	rts

;**************************************************
robcol
;translate Y into value for collision table
	lda	ytst
	clc
	adc	#4		;offset for actual floor
	and	#$f8
	tay
	cmp	#198		;do not process if off screen
	bcs	..out
;get x for use as an index
	lda	scanlo,y	;use scan table to find collision position
	sta	pt1
	lda	scanhi,y
	clc
	adc	#$04
	sta	pt1+1
	lda	xtst
	lsr	a
	tay
	sta	rd_main
	lda	(pt1),y       ;get value at the pt1
	sta	rd_aux
	jmp	..cont
..out	lda	#0		;off screen returns 0
..cont	rts

;**************************************************
loclft
;find which lift we are standing on
	stx	xtst
	sta	rd_main
	lda	#0
	sta	rlftnum
	ldx	#0
lfsclp
	lda	ytmp		;convert graphic Y to character Y
	adc	#4
	lsr	a
	lsr	a
	lsr	a
	ldy	lifts+2 	;check H/V
	beq	hyplt
	ldy	lifts+4,x	;total number of platforms for lift
	jmp	platlp
hyplt	ldy	#1		;1 vertical platform for horizontal
platlp
	cmp	lifts+1,x	;check Y coordinate of lift
	beq	plfnd		;exit if found
	sec
	sbc	lifts+5,x	;subtract interplatform dist (from dork Yc)
	dey
	bne	platlp		;try next platform
	jmp	nxsclf		;try next lift if not this one
plfnd
	lda	xtmp		;confirmed Y coordinate, now check X
	lsr	a		;graphic to character
;horizontal platforms....
	ldy	lifts+2 	;check H/V
	bne	hxplt
	ldy	lifts+4,x	;total number of platforms for lift
	jmp	pltxlp
hxplt	ldy	#1		;1 vertical platform for horizontal
pltxlp
;	cmp	lifts,x 	;test each of the three characters
;	beq	fndlft		; comprising a lift - found on any match
	sec
	sbc	#1
	cmp	lifts,x
	beq	fndlft
;	sec
;	sbc	#1
;	cmp	lifts,x
;	beq	fndlft

	sec
	sbc	lifts+5,x	;subtract interplatform dist (from dork Xc)
	clc
	adc	#2
	dey
	bne	pltxlp		;try next platform

nxsclf				;not found - index (18 bytes) to next lift
	txa
	clc
	adc	#18
	tax
	inc	rlftnum		;next lift number
	lda	rlftnum
	cmp	n_lifts 	;at end?
	bne	lfsclp
;lift not found - wierd, for now "forget" we are on a lift
;**********
;KLUDGE
; this will be called each time - the above will be called each
; time and rely on this to correct this
	jmp	noflft1
fndlft
	ldx	xtst
	lda	#$ff
	jmp	..llres
noflft

noflft1
	ldx	xtst
	lda	#0
..llres
	sta	rd_aux
	rts

;**************************************************
cntrbot
	stx	xtst
	lda	rlftnum 	;get number and index (18 bytes/lift)
	asl	a
	sta	ctmp
	asl	a
	asl	a
	asl	a
	adc	ctmp
	tax
	sta	rd_main
	sta	wr_main
	lda	lifts+14,x	;don't center if vert lift
	beq	..noshft

	lda	lifts,x
	jmp	..shft

..noshft
	lda	#$ff
..shft	sta	rd_aux
	sta	wr_aux
	ldx	xtst
	rts

;**************************************************
;returns moving flag (a)
strtlft
	sta	ytst
	stx	xtst
	lda	rlftnum 	;get number and index (18 bytes/lift)
	asl	a
	sta	ctmp
	asl	a
	asl	a
	asl	a
	adc	ctmp
	tax
	sta	rd_main
	sta	wr_main
	ldy	#0
	lda	lifts+14,x	;don't start move if in motion
	bne	llfimv

	lda	ytst
	bne	gothr

	lda	lifts+3,x	;don't move right if at right end
	clc
	adc	lifts+4,x
	cmp	lifts+7,x
	beq	gothr1

gothr2
	lda	lifts+2,x	;ignore horizontal lifts here
	beq	..gthr2x
	lda	lifts+1,x	;*** prevent robot from exiting room
	cmp	#24
	beq	gothr1
..gthr2x
	inc	lifts+3,x
	lda	#$1
	sta	lifts+14,x
	jmp	llfimv
gothr
	lda	lifts+3,x	;don't move left if at left end
	beq	gothr2

gothr1
;	lda	lifts,x		;*** prevent robot from exiting room up
;	beq	gothr1

	dec	lifts+3,x
	lda	#$ff		;01
	sta	lifts+14,x
llfimv
	ldy	lifts+2,x	;ignore horizontal lifts here
	sta	rd_aux
	sta	wr_aux
	ldx	xtst
	rts

dorobot	ENDPROC
	end
