$CHIP(6502)
$PAGINATE
;$ERRONLY
;
; Pseudo digital joystick routine for the apple (for analog joysticks)
; By Richard T. Unruh
;

	 %INCLUDE prmseg.inc
	 %INCLUDE macro.inc

	 EXTERN   FIRE0,FIRE1,KEY_DAT,KEY_DOWN,soundflag
	 EXTERN   ANRES, ANIN0, stick_flag,potcnt,rd_main,wr_main

	 extern   pot0_lo_thresh,pot0_hi_thresh
	 extern   pot1_lo_thresh,pot1_hi_thresh
	 extern   max_stick_val

	 PUBLIC   STICK, stkval,read_stick

	SEG	page0

	double	TDBL1
	double	TDBL2
	single	stkval
	single	temp1
JOY_TEMP1	 DS	1


	 SEG	  maincode

STICK	 PROC
	sta	  rd_main
	sta	  wr_main

	 lda	  stkval
	 sta	  oldstkval

	LDA	FIRE0
	AND	#$80
	STA	STKVAL
	LDA	FIRE1
	AND	#$80
	LSR	A
	ORA	STKVAL
	STA	STKVAL
;
; CHECK TO SEE IF HE PRESSED A KEY
;
;This patch added by HES, so bug him if it doesn't work.
;
	 LDA	  LAST_KEY    ;See if we're processing a repeated key.
	 BNE	  HAHA01      ;Jump if yes.
;
	 LDA	  KEY_DAT     ;Check KEY_DAT to see if they've hit
	 BPL	  HAHA02      ; a new key.  Jump if no.
;
	 AND	  #$7F	      ;Get rid of the high bit
	 STA	  LAST_KEY    ; and store the key in LAST_KEY to flag
			      ; that we have a key that may repeat.
	 STA	  KEY_DOWN    ;Clr the strobe flag.
	 JMP	  HAHA03      ;Jump to the end of the patch.
;
HAHA01			   ;This section processes repeated keys.
	 LDA	  KEY_DOWN    ;Are they still holding down a key?
	 BPL	  HAHA02      ;Jump if no.
	 LDA	  KEY_DAT     ;If so, get the key they're pressing.
	 CMP	  LAST_KEY    ;Is it the same key as last time?
	 BEQ	  HAHA03      ;If yes, jump to the end of the patch.
HAHA02
	 LDA	  #0	      ;Flag that there is no repeated key.
	 STA	  LAST_KEY
HAHA03
;
;End of patch.


	JSR	KEYDIR			;GET "STICK" VALUE
	BMI	DO_J_READ		;IF NEG, NOT VALID
	ORA	STKVAL			;MASK THE "STICK" VALUE IN
	STA	STKVAL
	JMP	fire_deboun

DO_J_READ
	lda	STICK_FLAG		;JOYSTICK PRESENT ?
	bmi	fire_deboun		;nope

	JSR	READ_STICK

;
; AT THIS POINT, THE "DIGITIZED" VALUES FOR THE POT INPUTS ARE IN POTCNT0-3.
; THE VALUES ARE NOT USABLE BY A ROUTINE NEEDING THE FULL ANALOG CONTROL
; DUE TO THE ROUTINE RUNNING FOR ONLY 2 MICROSECONDS (A "FULL" READ TAKES 3).
;

;
; ANALOG TO DIGITAL CONVERSION:
; RETURNED IN stkval LIKE THIS: (IN ALL CASES, LOGIC LEVEL 1 IS ACTIVE)
;
;	7 6 5 4 3 2 1 0
;	X				FIRE BUTTON 0
;	  X				FIRE BUTTON 1
;	    ?				DON'T CARE
;	      ? 			DON'T CARE
;		X			JOYSTICK RIGHT
;		  X			JOYSTICK LEFT
;		    X			JOYSTICK DOWN
;		      X 		JOYSTICK UP
;
;
;

	LDA	POTCNT
	CMP	POT0_LO_THRESH
	BCC	ITSLOW0
	CMP	POT0_HI_THRESH
	BCC	ITSMID0
	LDA	#$08
	BNE	SETSTICK0
ITSLOW0
	LDA	#$04
	BNE	SETSTICK0
ITSMID0
	LDA	#$00
SETSTICK0
	ORA	STKVAL
	STA	STKVAL


	LDA	POTCNT+1
	CMP	POT1_LO_THRESH
	BCC	ITSLOW1
	CMP	POT1_HI_THRESH

	BCC	ITSMID1
	LDA	#$02
	BNE	SETSTICK1
ITSLOW1
	LDA	#$01
	BNE	SETSTICK1
ITSMID1
	LDA	#$00
SETSTICK1
	ORA	STKVAL
	STA	STKVAL

;
; DEBOUNCE THE FIRE BUTTONS
;
FIRE_DEBOUN
	LDA	FIRE0		;CHECK FOR FIRE0 DEBOUNCE
	AND	#$80
	STA	JOY_TEMP
	LDA	STKVAL
	AND	#$80
	EOR	JOY_TEMP
	BEQ	CK_FIRE1

	LDA	STKVAL		;FIRE0 IS INVALID
	AND	#$4F
	STA	STKVAL
CK_FIRE1
	LDA	FIRE1		;CHECK FOR FIRE1 DEBOUNCE
	AND	#$80
	LSR	A
	STA	JOY_TEMP
	LDA	STKVAL
	AND	#$40
	EOR	JOY_TEMP
	BEQ	JOY_DONE

	LDA	STKVAL		;FIRE1 IS INVALID
	AND	#$8F
	STA	STKVAL






JOY_DONE
	lda	stkval
	ldx	last_key
	RTS

JOY_TEMP	 DS	1
LAST_KEY	  DS	   1

STICK	 ENDPROC

KEYDIR	 PROC

	AND	#$7F
	STA	TEMP1

	LDX	#$09
KEY_LP
	LDA	KEYASC_TAB,X
	CMP	TEMP1
	BEQ	KEY_FOUND
	DEX
	BPL	KEY_LP
	RTS
KEY_FOUND
	LDA	KEYVAL_TAB,X
	cmp	#$10
	beq	..toggle_sound
	ldx	  #$00
	rts
..toggle_sound
	lda	oldstkval
	and	#$10
	bne	  ..same_as_last_time
	lda	  #$80
	eor	soundflag
	sta	soundflag

..same_as_last_time
	 lda	  #$10
	RTS


KEYASC_TAB
	 DB	"UIOLNK,J ",$13 	     ;^S FOR SOUND TOGGLER


KEYVAL_TAB
	 DB	5,1,9,8,6,2,10,4,0,$10

oldstkval
	 ds	  1

KEYDIR	 ENDPROC













READ_STICK	  PROC

	LDA	#$00		;RESET ALL POT COUNTERS
	STA	POTCNT
	STA	POTCNT+1

	LDY	MAX_STICK_VAL	;LOOP FOR MAXIMUM STICK VAL*0.6666666
				;(INITIALLY SET TO 256 ITERATIONS)

	LDA	ANRES		;RESET ALL POT TIMERS

JOYLOOP
	LDX	#$01		;97 CLOCK CYCLES
CKIN
	LDA	ANIN0,X 	;TEST FOR CAP DISCHARGE
	BPL	INLOW		;THIS ONE'S DONE
	INC	POTCNT,X	;INC THE COUNT VALUE
	JMP	JLPDONE 	;BRANCH TO LOOP LOGIC
INLOW
	NOP			;WAIT 10 CYCLES TO MAKE TIMING THE SAME
	NOP
	NOP
	NOP
	NOP

JLPDONE
	DEX			;DEC THE POT POINTER
	BPL	CKIN

	DEY
	BNE	JOYLOOP

	RTS

READ_STICK	  ENDPROC

	 END

